{###############################################################################
#    Scintilla component for HiAsm                                             #
#    *****************************                                             #
#                                            #
#     Scintilla - http://www.scintilla.org/                              #
#    -----------------------------------------------                           #
#     - andrestudio (andrestudio@list.ru)                            #
#    12.05.2013                                                                #
###############################################################################}

unit hiScintilla;

interface
 
uses Kol, Share, Win, Windows, Messages, SciLexer, SciSupport;

const
  SCI_FOLDALL        = 2662;
  SCI_FOLDLINE       = 2237;
  SCI_SETTECHNOLOGY  = 2630;
  SCI_SETFONTQUALITY = 2611;
  SCN_FOCUSOUT       = 2029;
  SCI_FOLDCHILDREN   = 2238;
  brackets           = ['(', ')', '[', ']', '{', '}'];

  Default     = SC_CHARSET_DEFAULT;
  Utf8        = SC_CP_UTF8;
  Ansi        = SC_CHARSET_ANSI;
  Arabic      = SC_CHARSET_ARABIC;
  Baltic      = SC_CHARSET_BALTIC;
  ChineseBig5 = SC_CHARSET_CHINESEBIG5;
  ChineseGb   = SC_CHARSET_GB2312;
  Cyrillic    = SC_CHARSET_CYRILLIC;
  Easteurope  = SC_CHARSET_EASTEUROPE;
  Greek       = SC_CHARSET_GREEK;
  Hangul      = SC_CHARSET_HANGUL;
  Hebrew      = SC_CHARSET_HEBREW;
  Japanese    = SC_CHARSET_SHIFTJIS;
  Korean      = SC_CHARSET_JOHAB;
  Mac         = SC_CHARSET_MAC;
  Oem         = SC_CHARSET_OEM;
  Russian     = SC_CHARSET_RUSSIAN;
  Symbol      = SC_CHARSET_SYMBOL;
  Thai        = SC_CHARSET_THAI;
  Turkish     = SC_CHARSET_TURKISH;
  Vietnamese  = SC_CHARSET_VIETNAMESE;
  Westeurope  = SC_CHARSET_8859_15;

  Circle     = SC_MARK_CIRCLE;
  Roundrect  = SC_MARK_ROUNDRECT;
  Arrow      = SC_MARK_ARROW;
  Smallrect  = SC_MARK_SMALLRECT;
  Shortarrow = SC_MARK_SHORTARROW;
  Arrowdown  = SC_MARK_ARROWDOWN;
  Minus      = SC_MARK_MINUS;
  Plus       = SC_MARK_PLUS;
  Arrows     = SC_MARK_ARROWS;
  Flag       = MARKER_MAX;  
  BACKGROUND  = SC_MARK_BACKGROUND;
  UNDERLINE  = SC_MARK_UNDERLINE;
  LEFTRECT = SC_MARK_LEFTRECT;
  DOTDOTDOT = SC_MARK_DOTDOTDOT;
  
  PLAIN = INDIC_PLAIN;
  SQUIGGLE = INDIC_SQUIGGLE;
  TT = INDIC_TT;
  DIAGONAL = INDIC_DIAGONAL;
  STRIKE = INDIC_STRIKE;
  BOX = INDIC_BOX;
  ROUNDBOX = INDIC_ROUNDBOX;
  DASH = INDIC_DASH;
  DOTBOX = INDIC_DOTBOX;
  TEXTFORE = 17;
  
  HIDDEN = ANNOTATION_HIDDEN;
  STANDARD = ANNOTATION_STANDARD;
  BOXED = ANNOTATION_BOXED;
  INDENTED = 3;

  fCircle     = 0;
  fBox        = 1;
  fSimplified = 2;

  wrapWord       = 1;
  wrapChar       = 2;
  wrapWhitespace = 3;

  ArrCodepage: array[0..21] of integer =
    (1, 65001, 0, 178, 186, 136, 134, 1251, 238, 161, 129, 177,
    128, 130, 77, 255, 204, 2, 222, 162, 163, 1000);

  ArrMarkerForm: array[0..13] of integer =
    (0, 1, 2, 3, 4, 6, 7, 8, 24, 31, 22, 29, 27, 23);
    
  ArrIndicForm: array[0..9] of integer =
    (0, 1, 2, 3, 4, 9, 6, 7, 12, 17);
  ArrAnnotationStyle: array[0..3] of integer =
    (0, 1, 2, 3);	    
	
type
  THIScintilla = class(THIWin)
  private
    FSciDllHandle: HMODULE;
    fFontLstStyles: PStrList;
    fTypeStyles:  PStrList;
    Arr:          PArray;
    flinecount:   integer;    
    fposition:    integer;
    posSearch:    integer;
    strSearch:    string;
    modSearch:    string;
    pos0:         boolean;
    rePos:        integer;
    Styleback_34_35: integer;
    Markersymbol: integer;

    procedure _Set(var Item: TData; var Val: TData);
    function _Get(var Item: TData; var Val: TData): boolean;
    function _Count: integer;
    procedure _Add(var Val: TData);

//    procedure LoadSciLibrary;
//    procedure FreeSciLibrary;
    procedure CreateWnd;
    function CP(msgcode: DWORD; wParam, lParam: integer): integer;
    procedure SetStyleLang(var lst: PStrList);
    procedure BraceLight;
    procedure AutoSetMarginNumberWidth;
    procedure SetScrollBars(idx: integer);
    procedure SetFoldIconColorFore(Colour: integer);
    procedure SetFoldIconColorBack(Colour: integer);
    procedure SetFoldIcons(TypeIcons: integer);

  public
    fwordstring: string;
    fs:          string;
    fchr:        char;

    _event_onError:         THI_Event;
	_event_onLoad:          THI_Event;
	_event_onSave:          THI_Event;
	_event_onClear:         THI_Event;
    _event_onSearch:        THI_Event;
    _event_onChange:        THI_Event;
    _event_onGetLineMarker: THI_Event;
    _event_onGetLineFold:   THI_Event;   
    
    _data_str:         THI_Event;
    _data_FileName:    THI_Event;
    _data_TextTarget:  THI_Event;
    _data_PosSearchStart: THI_Event;
    _data_SearchFlags: THI_Event;
    _data_Codepage:    THI_Event;
    _data_ArrayFold:   THI_Event;
    _data_TextAnnotation:  THI_Event;

    _prop_ScrollBars:   byte;
    _prop_ReadOnly:     byte;
    _prop_Strings:      string;
    _prop_FileName:     string;
    _prop_AddType:      integer;
    _prop_Styles:       string;
    _prop_Codepage:     cardinal;
    _prop_WordsKey1:    string;
    _prop_WordsKey2:    string;
    _prop_WordsKey3:    string;
    _prop_WordsKey4:    string;
    _prop_CaretLineVisible: boolean;
    _prop_CaretLineBack: integer;
    _prop_SelectBack:   integer;
    _prop_SelectFore:   integer;
    _prop_BraceHighlight: boolean;
    _prop_BraceFore:    integer;
    _prop_BraceBold:    boolean;
    _prop_BraceBadFore: integer;
    _prop_BraceBadBold: boolean;
    _prop_IndentationGuides: boolean;
    _prop_ColorIGFore:  integer;
    _prop_VLines:       boolean;
    _prop_ColorVLines:  integer;
    _prop_LongVLines:   integer;
    _prop_ColorOfFlags: integer;
    _prop_Numbering:    boolean;
	_prop_NumSizeCorrect:integer;
    _prop_MarginNumFore: integer;
    _prop_MarginNumBack: integer;
    _prop_Markers:      boolean;
    _prop_MarkMarginWidth: integer;
    _prop_MarkerForm:   cardinal;
    _prop_MarkerBack:   integer;
    _prop_MarkerFore:   integer;
    _prop_Fold:         boolean;
    _prop_FoldMarginWidth: integer;
    _prop_FoldMarginColor1: integer;
    _prop_FoldMarginColor2: integer;
    _prop_FoldIcon:     cardinal;
    _prop_FoldIconFore: integer;
    _prop_FoldIconBack: integer;
    _prop_AutoCompStrings: string;
    _prop_AutoComplete: boolean;
    _prop_MinWordLen:   integer;
    _prop_IgnoreCase:   boolean;
    _prop_WrapMode:     cardinal;
    _prop_WrapVisualFlags: boolean;
    _prop_UserPoupMenu: integer;
    _prop_IndicStyle:   cardinal;
    _prop_IndicFore:   integer;
    _prop_SetIndicatorCurrent: integer;
    _prop_IndicatorAdd: integer;
    _prop_AnnotationBack: integer;
    _prop_AnnotationFore: integer;
    _prop_AnnotationStyle: integer;

    constructor Create(Parent: PControl);
    destructor Destroy; override;
    procedure Init; override;

    procedure _work_doAdd(var _Data: TData; Index: word);
    procedure _work_doClear(var _Data: TData; Index: word);
    procedure _work_doText(var _Data: TData; Index: word);
    procedure _work_doSave(var _Data: TData; Index: word);
    procedure _work_doLoad(var _Data: TData; Index: word);
	procedure _work_doCodepage(var _Data: TData; Index: word);
    procedure _work_doSetSelect(var _Data: TData; Index: word);
    procedure _work_doSetSelStart(var _Data: TData; Index: word);
    procedure _work_doSetSelLength(var _Data: TData; Index: word);
    procedure _work_doSetZoom(var _Data: TData; Index: word);
    procedure _work_doEnsureVisible(var _Data: TData; Index: word);
    procedure _work_doStyles(var _Data: TData; Index: word);
    procedure _work_doScrollBars(var _Data: TData; Index: word);
    procedure _work_doReadOnly(var _Data: TData; Index: word);
    procedure _work_doSearch(var _Data: TData; Index: word);
    procedure _work_doCaretlinevisible(var _Data: TData; Index: word);
    procedure _work_doCaretLineBack(var _Data: TData; Index: word);
    procedure _work_doBraceHighlight(var _Data: TData; Index: word);
    procedure _work_doBraceFore(var _Data: TData; Index: word);
    procedure _work_doBraceBold(var _Data: TData; Index: word);
    procedure _work_doBraceBadFore(var _Data: TData; Index: word);
    procedure _work_doBraceBadBold(var _Data: TData; Index: word);
    procedure _work_doSelectBack(var _Data: TData; Index: word);
    procedure _work_doSelectFore(var _Data: TData; Index: word);
    procedure _work_doColor(var _Data: TData; Index: word);
    procedure _work_doFont(var _Data: TData; Index: word);
    procedure _work_doIndentationGuides(var _Data: TData; Index: word);
    procedure _work_doColorIGFore(var _Data: TData; Index: word);
    procedure _work_doVLines(var _Data: TData; Index: word);
    procedure _work_doLongVLines(var _Data: TData; Index: word);
    procedure _work_doColorVLines(var _Data: TData; Index: word);
    procedure _work_doColorOfFlags(var _Data: TData; Index: word);
    procedure _work_doNumbering(var _Data: TData; Index: word);
    procedure _work_doMarginNumFore(var _Data: TData; Index: word);
    procedure _work_doMarginNumBack(var _Data: TData; Index: word);
    procedure _work_doMarkers(var _Data: TData; Index: word);
    procedure _work_doMarkMarginWidth(var _Data: TData; Index: word);
    procedure _work_doMarkerBack(var _Data: TData; Index: word);
    procedure _work_doMarkerFore(var _Data: TData; Index: word);
    procedure _work_doMarkerForm(var _Data: TData; Index: word);
    procedure _work_doMarkerAdd(var _Data: TData; Index: word);
    procedure _work_doMarkerDelete(var _Data: TData; Index: word);
    procedure _work_doMarkerDeleteAll(var _Data: TData; Index: word);
    procedure _work_doGetLineMarker(var _Data: TData; Index: word);
    procedure _work_doGetLineMarkerNext(var _Data: TData; Index: word);
    procedure _work_doFold(var _Data: TData; Index: word);
    procedure _work_doFoldMarginWidth(var _Data: TData; Index: word);
    procedure _work_doFoldMarginColor1(var _Data: TData; Index: word);
    procedure _work_doFoldMarginColor2(var _Data: TData; Index: word);
    procedure _work_doFoldIconFore(var _Data: TData; Index: word);
    procedure _work_doFoldIconBack(var _Data: TData; Index: word);
    procedure _work_doFoldAll(var _Data: TData; Index: word);
    procedure _work_doGetLineFold(var _Data: TData; Index: word);
    procedure _work_doSetLineFold(var _Data: TData; Index: word);
    procedure _work_doWrapMode(var _Data: TData; Index: word);
    procedure _work_doWrapVisualFlags(var _Data: TData; Index: word);
    procedure _work_doAutoComplete(var _Data: TData; Index: word);
    procedure _work_doAutoCompStrings(var _Data: TData; Index: word);
    procedure _work_doMinWordLen(var _Data: TData; Index: word);
    procedure _work_doIgnoreCase(var _Data: TData; Index: word);
    procedure _work_doUndoRedo(var _Data: TData; Index: word);
    procedure _work_doEditingSelected(var _Data: TData; Index: word);
	procedure _work_doSetEmptyUndoBuffer(var _Data: TData; Index: word);
	procedure _work_doGetStatus(var _Data: TData; Index: word);
    procedure _work_doIndicators(var _Data: TData; Index: word);
    procedure _work_doIndicStyle(var _Data: TData; Index: word);
    procedure _work_doIndicFore(var _Data: TData; Index: word);  
	procedure _work_doIndicatorAdd(var _Data: TData; Index: word);  
	procedure _work_doSetIndicatorCurrent(var _Data: TData; Index: word);
	procedure _work_doIndicatorClear(var _Data: TData; Index: word);
	procedure _work_doAnnotationBack(var _Data: TData; Index: word);
    procedure _work_doAnnotationFore(var _Data: TData; Index: word);
    procedure _work_doAnnotationStyle(var _Data: TData; Index: word);
    procedure _work_doAnnotationSetText(var _Data: TData; Index: word);
    procedure _work_doAnnotationStrDel(var _Data: TData; Index: word);
    procedure _work_doAnnotationClearAll(var _Data: TData; Index: word);
	                                                         
    procedure _var_Text(var _Data: TData; Index: word);
    procedure _var_Count(var _Data: TData; Index: word);
    procedure _var_Array(var _Data: TData; Index: word);
    procedure _var_Position(var _Data: TData; Index: word);
    procedure _var_SelText(var _Data: TData; Index: word);
    procedure _var_LinePosition(var _Data: TData; Index: word);
    procedure _var_StrPosition(var _Data: TData; Index: word);
    procedure _var_TextLength(var _Data: TData; Index: word);
	procedure _var_GetModify(var _Data: TData; Index: word);
  end;

implementation

//   .
procedure THIScintilla.AutoSetMarginNumberWidth;
var
  len, i: integer;
  lenStr: string;
begin
  LenStr := '';
  len := CP(SCI_LINEFROMPOSITION, CP(SCI_GETTEXTLENGTH, 0, 0), 0);
  len := Length(int2str(len)); // 
  for i := 0 to len - 1 do
    lenStr := lenStr + '0';
  //         '_0000' 
  //(SCI_SetMarginWidthN,0,'_000'),     
  CP(SCI_SetMarginWidthN, 0, CP(SCI_TEXTWIDTH, STYLE_LINENUMBER,
    longint(PChar(lenStr)))); //longint(PChar('_' + lenStr)))); 
end;

procedure THIScintilla.SetScrollBars(idx: integer);
begin
  case idx of
    0:
    begin
      CP(SCI_SETHSCROLLBAR, 0, 0);
      CP(SCI_SETVSCROLLBAR, 0, 0);
      CP(SCI_SETWRAPMODE, _prop_WrapMode, 0); //    .     
      if _prop_WrapVisualFlags then           // /   .
        CP(SCI_SETWRAPVISUALFLAGS, SC_WRAPVISUALFLAG_START or SC_WRAPVISUALFLAG_END, 0);
    end;
    1:
    begin
      CP(SCI_SETHSCROLLBAR, 1, 0);
      CP(SCI_SETVSCROLLBAR, 0, 0);
      CP(SCI_SETWRAPMODE, 0, 0); //    .
    end;
    2:
    begin
      CP(SCI_SETHSCROLLBAR, 0, 0);
      CP(SCI_SETVSCROLLBAR, 1, 0);
      CP(SCI_SETWRAPMODE, _prop_WrapMode, 0); //   .      
      if _prop_WrapVisualFlags then           // /   .
	    CP(SCI_SETWRAPVISUALFLAGS, SC_WRAPVISUALFLAG_START or SC_WRAPVISUALFLAG_END, 0);
    end;
    3:
    begin
      CP(SCI_SETHSCROLLBAR, 1, 0);
      CP(SCI_SETVSCROLLBAR, 1, 0);
      CP(SCI_SETWRAPMODE, 0, 0); //    .
    end;
  end;
end;

procedure THIScintilla.BraceLight; //  .
var
  pos: integer;       //  
  posMatch: integer;  //   
  // cPos-  , cRight- , cLeft- 
  cPos, cRight, cLeft: char;
begin
  pos := CP(SCI_GETCURRENTPOS, 0, 0);      //   
  //           - 
  // -      
  if rePos <> pos then
  begin
    CP(SCI_BRACEHIGHLIGHT, -1, -1);        //    
    cPos  := Chr(CP(SCI_GETCHARAT, pos, 0));
    cRight := Chr(CP(SCI_GETCHARAT, pos + 1, 0));
    cLeft := Chr(CP(SCI_GETCHARAT, pos - 1, 0));

    if cLeft in brackets then
    begin
      posMatch := CP(SCI_BRACEMATCH, pos - 1, 0);
      if posMatch <> -1 then
        CP(SCI_BRACEHIGHLIGHT, pos - 1, posMatch)
      else
        CP(SCI_BRACEBADLIGHT, pos - 1, 0);
    end
    else
    if cPos in brackets then
    begin
      posMatch := CP(SCI_BRACEMATCH, pos, 0);
      if posMatch <> -1 then
        CP(SCI_BRACEHIGHLIGHT, pos, posMatch)
      else
        CP(SCI_BRACEBADLIGHT, pos, 0);
    end
    else
    if cRight in brackets then
    begin
      posMatch := CP(SCI_BRACEMATCH, pos + 1, 0);
      if (posMatch <> -1) and (CP(SCI_BRACEMATCH, pos, 0) <> -1) then
        CP(SCI_BRACEHIGHLIGHT, pos + 1, posMatch)
      else
      if CP(SCI_BRACEMATCH, pos, 0) <> -1 then
        CP(SCI_BRACEBADLIGHT, pos + 1, 0);
    end;
    rePos := pos;
  end;
end;

procedure THIScintilla.SetFoldIconColorFore(Colour: integer);
begin
  CP(SCI_MarkerSetFore, SC_MARKNUM_FOLDEROPEN, Colour);    // , 
  CP(SCI_MarkerSetFore, SC_MARKNUM_FOLDER, Colour);        // , 
  CP(SCI_MarkerSetFore, SC_MARKNUM_FOLDEREND, Colour);     // -, 
  CP(SCI_MarkerSetFore, SC_MARKNUM_FOLDEROPENMID, Colour); // -, 
end;

procedure THIScintilla.SetFoldIconColorBack(Colour: integer);
begin
  CP(SCI_MarkerSetBack, SC_MARKNUM_FOLDEROPEN, Colour);    // , 
  CP(SCI_MarkerSetBack, SC_MARKNUM_FOLDER, Colour);        // , 
  CP(SCI_MarkerSetBack, SC_MARKNUM_FOLDERSUB, Colour);     //
  CP(SCI_MarkerSetBack, SC_MARKNUM_FOLDERTAIL, Colour);    //
  CP(SCI_MarkerSetBack, SC_MARKNUM_FOLDEREND, Colour);     // -, 
  CP(SCI_MarkerSetBack, SC_MARKNUM_FOLDEROPENMID, Colour); // -, 
  CP(SCI_MarkerSetBack, SC_MARKNUM_FOLDERMIDTAIL, Colour); // -
end;

procedure THIScintilla.SetFoldIcons(TypeIcons: integer);
begin
  case TypeIcons of
    0:
    begin //  
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDEROPEN, SC_MARK_CIRCLEMINUS);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDER, SC_MARK_CIRCLEPLUS);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDERSUB, SC_MARK_VLINE);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDERTAIL, SC_MARK_LCORNERCURVE);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDEREND, SC_MARK_CIRCLEPLUSCONNECTED);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDEROPENMID, SC_MARK_CIRCLEMINUSCONNECTED);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_TCORNERCURVE);
    end;
    1:
    begin //   
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDEROPEN, SC_MARK_BOXMINUS);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDER, SC_MARK_BOXPLUS);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDERSUB, SC_MARK_VLINE);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDERTAIL, SC_MARK_LCORNER);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDEREND, SC_MARK_BOXPLUSCONNECTED);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDEROPENMID, SC_MARK_BOXMINUSCONNECTED);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_TCORNER);
    end;
    2:
    begin// ,  
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDEROPEN, SC_MARK_MINUS);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDER, SC_MARK_PLUS);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDERSUB, SC_MARK_EMPTY);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDERTAIL, SC_MARK_EMPTY);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDEREND, SC_MARK_EMPTY);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDEROPENMID, SC_MARK_EMPTY);
      CP(SCI_MarkerDefine, SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_EMPTY);
    end;
  end;
end;

//ScintillaEvents
function ScintillaEvents(Sender: PControl; var Msg: TMsg; var Rslt: integer): boolean;
var
  NMhdr: PNMHdr;
  scn: PSCNotification;
  intchr, l: integer;
  sc: THIScintilla;
  s: string;
  LineClick, LevelClick: integer;
  SciCurrentPos: integer;
begin
  Result := False;  
  if (Msg.message = WM_CHAR) and (GetKeyState(9) < 0) then // TAB
  begin
    SendMessage(Sender.Handle, SCI_TAB, 0, 0);
    SendMessage(Sender.Handle, SCI_GRABFOCUS, 0, 0);
  end;
  
  if Msg.message = WM_NOTIFY then
  begin
    NMhdr := Pointer(Msg.lParam);
    if (NMhdr^.hwndFrom = Sender.Handle) then
    begin
      sc := THIScintilla(Sender.Tag);
      with sc do
      begin
        scn := PSCNotification(Msg.LParam);
        case NMhdr^.code of
          //SCN_DOUBLECLICK:_debug('DOUBLECLICK');
          //SCN_ZOOM:_debug('ZOOM');
          //SCN_CALLTIPCLICK: _debug('SCN_CALLTIPCLICK');
          //SCN_USERLISTSELECTION: _debug('SCN_USERLISTSELECTION');
          //_debug(PChar(scn^.text)); (scn^.ch)  ., . scisupport 24-40 
          SCN_CHARADDED:
           //     
            if _prop_AutoComplete and not boolean(CP(SCI_GETREADONLY, 0, 0)) then
            begin  
              SciCurrentPos := CP(SCI_GETCURRENTPOS, 0, 0);
              if not ((_prop_MinWordLen > 1) and (SciCurrentPos + 1 <=
                _prop_MinWordLen)) then
                CP(SCI_AUTOCSHOW, _prop_MinWordLen, longint(PChar(_prop_AutoCompStrings)));
            end;
          SCN_AUTOCSELECTION:
            //             
            if _prop_AutoComplete then CP(SCI_INSERTTEXT, scn^.position, longint(PChar(scn^.Text)));
          SCN_MARGINCLICK:
          begin 
            //    
            if _prop_Markers and (scn^.Margin = 1) then
            begin
              LineClick := CP(SCI_LINEFROMPOSITION, scn^.position, 0);
              case CP(SCI_MARKERGET, LineClick, 0) of
                0: CP(SCI_MARKERADD, LineClick, 0);
                1: CP(SCI_MARKERDELETE, LineClick, 0);
              end;
            end;
            if _prop_Fold and (scn^.Margin = 2) then
            begin
              LineClick := CP(SCI_LINEFROMPOSITION, scn^.position, 0);
              CP(SCI_TOGGLEFOLD, LineClick, 0);
             // / ,  SHIFT+CTRL+   , .
             // if (scn^.modifiers and SCMOD_SHIFT <> 0) and (scn^.modifiers and SCMOD_CTRL <> 0) then
             // CP(SCI_FOLDALL, 2, 0); 
            end;
          end;
          SCN_UPDATEUI: if _prop_BraceHighlight then BraceLight; //    .
          SCN_MODIFIED:   // onChange.
          begin 
            l := CP(SCI_GETTEXTLENGTH, 0, 0);
            SetLength(s, l + 1);
            CP(SCI_GETTEXT, l + 1, longint(PChar(s)));
            _hi_OnEvent(_event_onChange, (PChar(s)));
            if _prop_Numbering then AutoSetMarginNumberWidth; 
          end;
        end;
      end;
    end;
  end;
end;

constructor THIScintilla.Create;
begin
  inherited Create(Parent);
  //  
//  Control := _NewControl(Fparent, 'SCINTILLA', WS_CHILD or WS_VISIBLE, false, nil);     
   Control := _NewCommonControl(Fparent, 'SCINTILLA', WS_CHILD or WS_VISIBLE, false, nil);
  fFontLstStyles := NewStrList;
  fTypeStyles := NewStrList;
end;

destructor THIScintilla.Destroy;
begin

end;

function THIScintilla.CP;
begin
  Result := Control.Perform(msgcode, wParam, lParam);
end;

//procedure THIScintilla.LoadSciLibrary;
//begin
//  FSciDllHandle := LoadLibrary('SciLexer.dll');
//  if FSciDllHandle = 0 then
//    _debug('SciLexer.DLL not load library');
//end;

//procedure THIScintilla.FreeSciLibrary;
//begin
//  if FSciDllHandle <> 0 then
//    try
//      FreeLibrary(FSciDllHandle);
//    finally
//      FSciDllHandle := 0;
//    end;
// end;

procedure THIScintilla.CreateWnd;
begin
  //  
//  LoadSciLibrary;
  //  Scintilla 
  CP(SCI_GETDIRECTFUNCTION, 0, 0);
  CP(SCI_GETDIRECTPOINTER, 0, 0);
  //CP(SCI_SETTECHNOLOGY, 2, 0); // 
end;

 //     
procedure THIScintilla.SetStyleLang;
var
  Name, dtkey, tmpstr, s: string;
  i, j, _WORDSTILING_: integer;
  paramStyles: array of string;

  function StrToBool(dt: string): integer;
  begin
    Result := 0;
    dt:= AnsiUpperCase(dt);
    if dt = 'TRUE' then Result := 1;
  end;

  procedure ParseStrStyles(s: string; const d1, d2: PChar);
  var
    subStr, rStr: string;
    i, len, posD1, posD2, Count, pN: integer;
  begin
    s  := Trim(s);
    s  := (s + d1);
    len := Length(s);
    posD1 := 1;
    pN := 0;
    for i := 1 to len do
    begin
      if s[i] = d1 then
      begin
        Inc(pN);
        SetLength(paramStyles, pN + 2);
        Count := i - posD1;
        Substr := Copy(s, posD1, Count);
        posD2 := Pos(d2, Substr);
        Count := Count - posD2;
        rStr  := Copy(Substr, posD2 + 1, Count);
        posD1 := i + 1;
        paramStyles[pN] := rStr;
      end;
    end;
  end;

begin
  for i := 0 to fFontLstStyles.Count - 1 do
  begin
    //  
    tmpstr := fFontLstStyles.Items[i];
    //    "//",    ,   .
    if (tmpstr = '') then
      continue;
    if (tmpstr[1] = '/') and (tmpstr[2] = '/') or (tmpstr[1] = ' ') then
      continue;

    //  .
    //Language=[1]
    if (tmpstr[1] = '@') and (tmpstr[2] = '1') then
    begin
      ParseStrStyles(tmpstr, ',', '=');
      CP(SCI_SETLEXER, str2int(paramStyles[1]), 0);
      continue;
    end;
	
   //  .
    if (tmpstr[1] = '@') and (tmpstr[2] = '2') then
    begin
      ParseStrStyles(tmpstr, ',', '=');
      CP(SCI_STYLESETBACK, STYLE_DEFAULT, str2int(paramStyles[1]));  
	  _prop_Color:= str2int(paramStyles[1]);
      continue;
    end;
	
    //    .
    if (tmpstr[1] = '@') and (tmpstr[2] = '3') then
    begin
      ParseStrStyles(tmpstr, ',', '=');
      CP(SCI_SETCARETLINEBACK, str2int(paramStyles[1]), 0);
      Styleback_34_35 := str2int(paramStyles[1]);
      continue;
    end;

    //      .
    if (tmpstr[1] = '@') and (tmpstr[2] = '4') then
    begin
      ParseStrStyles(tmpstr, ',', '=');
      CP(SCI_SETSELBACK, STYLE_DEFAULT, str2int(paramStyles[1]));
      CP(SCI_SETSELFORE, STYLE_DEFAULT, str2int(paramStyles[2]));
      continue;
    end;

    //          .
    if (tmpstr[1] = '@') and (tmpstr[2] = '5') then
    begin
      ParseStrStyles(tmpstr, ',', '=');
      //    34 -   
      CP(SCI_STYLESETBOLD, 34, StrToBool(paramStyles[1]));
      CP(SCI_STYLESETSIZE, 34, _prop_Font.size);
      if CP(SCI_GETCARETLINEVISIBLE, 0, 0) = 0 then
        CP(SCI_STYLESETBACK, 34, _prop_Color)
      else
        CP(SCI_STYLESETBACK, 34, CP(SCI_GETCARETLINEBACK, 0, 0));
      CP(SCI_STYLESETFORE, 34, str2int(paramStyles[2]));
      //    35 -   
      CP(SCI_STYLESETBOLD, 35, StrToBool(paramStyles[3]));
      CP(SCI_STYLESETSIZE, 35, _prop_Font.size);
      if CP(SCI_GETCARETLINEVISIBLE, 0, 0) = 0 then
        CP(SCI_STYLESETBACK, 35, _prop_Color)
      else
        CP(SCI_STYLESETBACK, 35, CP(SCI_GETCARETLINEBACK, 0, 0));
      CP(SCI_STYLESETFORE, 35, str2int(paramStyles[4]));
      continue;
    end;

    //    .
    if (tmpstr[1] = '@') and (tmpstr[2] = '6') then
    begin
      ParseStrStyles(tmpstr, ',', '=');
	  CP(SCI_STYLESETBACK, 37, _prop_Color);
      CP(SCI_STYLESETFORE, 37, str2int(paramStyles[1]));
      continue;
    end;

    //           .
    if (tmpstr[1] = '@') and (tmpstr[2] = '7') then
    begin
      ParseStrStyles(tmpstr, ',', '=');
      CP(SCI_SETEDGECOLOUR, str2int(paramStyles[1]), 0); 
      CP(SCI_STYLESETFORE, STYLE_DEFAULT, str2int(paramStyles[2]));	  
      continue;
    end;
	
    //      .
    if (tmpstr[1] = '@') and (tmpstr[2] = '8') then
    begin
      ParseStrStyles(tmpstr, ',', '=');
      CP(SCI_STYLESETFORE, STYLE_LINENUMBER, str2int(paramStyles[1]));  //  
      CP(SCI_STYLESETBACK, STYLE_LINENUMBER, str2int(paramStyles[2]));  //  
      CP(SCI_MARKERSETFORE, 0, str2int(paramStyles[3]));
      CP(SCI_MARKERSETBACK, 0, str2int(paramStyles[4]));	  
      continue;
    end;

    //    .
    if (tmpstr[1] = '@') and (tmpstr[2] = '9') then
    begin
      ParseStrStyles(tmpstr, ',', '=');
      CP(SCI_SETFOLDMARGINCOLOUR, 2, str2int(paramStyles[1]));   //  1
      CP(SCI_SETFOLDMARGINHICOLOUR, 2, str2int(paramStyles[2])); //  2
      SetFoldIconColorFore(str2int(paramStyles[3]));//    , 
      SetFoldIconColorBack(str2int(paramStyles[4]));//    , 	  
      continue;
    end;	
 			
    //         
    if (tmpstr[1] = '#') then
    begin
      ParseStrStyles(tmpstr, ',', '=');
      CP(SCI_STYLESETBOLD, str2int(paramStyles[1]), StrToBool(paramStyles[2]));
      CP(SCI_STYLESETITALIC, str2int(paramStyles[1]), StrToBool(paramStyles[3]));
      CP(SCI_STYLESETUNDERLINE, str2int(paramStyles[1]), StrToBool(paramStyles[4]));
      CP(SCI_STYLESETFORE, str2int(paramStyles[1]), Str2Int(paramStyles[5]));
	  CP(SCI_STYLESETBACK, str2int(paramStyles[1]), _prop_Color);
      if _prop_Codepage <> 65001 then
        CP(SCI_STYLESETCHARACTERSET, str2int(paramStyles[1]), _prop_Codepage);				
    end;
  end;
end;

procedure THIScintilla.Init;
//var
//  len, i: integer;
//  lenStr:PAnsiChar;
begin
  //  Scintilla  
  CreateWnd;  
  Control.tag := dword(Self);
  Control.AttachProc(ScintillaEvents); 
  inherited;
    
  if _prop_Codepage = 65001 then 
    CP(SCI_SETCODEPAGE, 65001, 0)
  else 
    CP(SCI_SETCODEPAGE, 0, 0); 

  //    
  CP(SCI_STYLESETFONT, STYLE_DEFAULT, longint(PChar(_prop_Font.Name)));
  CP(SCI_STYLESETSIZE, STYLE_DEFAULT, _prop_Font.size);
  CP(SCI_STYLESETBACK, STYLE_DEFAULT, _prop_Color); //   
  CP(SCI_STYLESETFORE, STYLE_DEFAULT, _prop_Font.Color); //  
  CP(SCI_STYLESETBOLD, STYLE_DEFAULT, _prop_Font.Style and 1);
  CP(SCI_STYLESETITALIC, STYLE_DEFAULT, _prop_Font.Style and 2);
  CP(SCI_STYLESETUNDERLINE, STYLE_DEFAULT, _prop_Font.Style and 4);
  
  CP(SCI_STYLECLEARALL, 0, 0);

  //         
  CP(SCI_STYLESETFORE, STYLE_DEFAULT, _prop_ColorOfFlags);

  // ********    *********************************************
  // /   .
  CP(SCI_SETCARETLINEVISIBLE, integer(_prop_CaretLineVisible), 0);
  CP(SCI_SETCARETLINEBACK, _prop_CaretLineBack, 0);//    
  CP(SCI_SETSELBACK, 1, _prop_SelectBack);         //   
  CP(SCI_SETSELFORE, 1, _prop_SelectFore);         //    
  //      ,       .
  CP(SCI_SETCARETLINEVISIBLEALWAYS, 1, 0);
  // *******************************************************************************

  // ********           ************
  if _prop_BraceHighlight then
  begin
    //    34 -   
    CP(SCI_STYLESETBOLD, 34, integer(_prop_BraceBold));
    CP(SCI_STYLESETSIZE, 34, _prop_Font.size);
    if CP(SCI_GETCARETLINEVISIBLE, 0, 0) = 0 then
      CP(SCI_STYLESETBACK, 34, _prop_Color)
    else
      CP(SCI_STYLESETBACK, 34, CP(SCI_GETCARETLINEBACK, 0, 0));
    CP(SCI_STYLESETFORE, 34, _prop_BraceFore);
    //    35 -   
    CP(SCI_STYLESETBOLD, 35, integer(_prop_BraceBadBold));
    CP(SCI_STYLESETSIZE, 35, _prop_Font.size);
    if CP(SCI_GETCARETLINEVISIBLE, 0, 0) = 0 then
      CP(SCI_STYLESETBACK, 35, _prop_Color)
    else
      CP(SCI_STYLESETBACK, 35, CP(SCI_GETCARETLINEBACK, 0, 0));
    CP(SCI_STYLESETFORE, 35, _prop_BraceBadFore);
  end;
  // *******************************************************************************

  // /   .
  CP(SCI_SETINDENTATIONGUIDES, integer(_prop_IndentationGuides), 0);
  CP(SCI_STYLESETBACK, STYLE_INDENTGUIDE, _prop_Color);
  CP(SCI_STYLESETFORE, STYLE_INDENTGUIDE, _prop_ColorIGFore);
  CP(SCI_SETINDENT, 4, 0); //     

  // / -  .
  CP(SCI_SETEDGEMODE, integer(_prop_VLines), 0);
  CP(SCI_SETEDGECOLOUR, _prop_ColorVLines, 0); //  
  CP(SCI_SETEDGECOLUMN, _prop_LongVLines, 0);  //   

  //      
  SetScrollBars(_prop_ScrollBars);

  // Margins
  //*****************************************************************************    
  //  
  CP(SCI_SetMarginTypen, 0, SC_MARGIN_NUMBER);
  CP(SCI_STYLESETFORE, STYLE_LINENUMBER, _prop_MarginNumFore);  //  
  CP(SCI_STYLESETBACK, STYLE_LINENUMBER, _prop_MarginNumBack);  //      
  CP(SCI_STYLESETBOLD, STYLE_LINENUMBER, 0);
  CP(SCI_STYLESETSIZE, STYLE_LINENUMBER, _prop_Font.size +_prop_NumSizeCorrect);
  
  //  
  CP(SCI_SetMarginTypen, 1, SC_MARGIN_SYMBOL);
      if not _prop_Fold then CP(SCI_SetMarginMaskN, 1, -1); 
//   ...
  CP(SCI_SetMarginSensitiveN, 1, 1);
  CP(SCI_MARKERSETFORE, 0, _prop_MarkerFore);
  CP(SCI_MARKERSETBACK, 0, _prop_MarkerBack);    
  Markersymbol := _prop_MarkerForm; 
  if _prop_Markers then
  begin
    CP(SCI_SetMarginWidthN, 1, _prop_MarkMarginWidth);
    CP(SCI_MARKERDEFINE, 0, _prop_MarkerForm); 
  end
  else
  begin
    CP(SCI_SetMarginWidthN, 1, 0);
    CP(SCI_MARKERDEFINE, 0, 5);   
  end;
  //
  CP(SCI_INDICSETSTYLE, 0, _prop_IndicStyle);
  CP(SCI_INDICSETFORE, 0, _prop_IndicFore); 
  //
  CP(SCI_STYLESETBACK, 25,_prop_AnnotationBack);
  CP(SCI_STYLESETFORE, 25,_prop_AnnotationFore);
    
  //  
  CP(SCI_SetMarginMaskN, 2, SC_MASK_FOLDERS);
  if _prop_Fold then CP(SCI_SetMarginWidthN, 2, _prop_FoldMarginWidth);
  CP(SCI_SetMarginSensitiveN, 2, 1);
  CP(SCI_SETFOLDMARGINCOLOUR, 2, _prop_FoldMarginColor1);   //  1
  CP(SCI_SETFOLDMARGINHICOLOUR, 2, _prop_FoldMarginColor2); //  2
  //CP(SCI_SETMARGINLEFT   , 2, 5);        //    ,         
  SetFoldIcons(_prop_FoldIcon);            //    
  if  _prop_FoldIcon <> 2 then
  begin
    SetFoldIconColorFore(_prop_FoldIconFore);//    , 
    SetFoldIconColorBack(_prop_FoldIconBack);//    , 
  end
  else
  begin                                      //    fSimplified
    SetFoldIconColorFore(_prop_FoldIconBack);//    , 
    SetFoldIconColorBack(_prop_FoldIconFore);//    , 
  end;    
  CP(SCI_SETFOLDFLAGS, 16, 0); //     ,  
  // The following is to handle SC_MOD_CHANGEFOLD in the Modified event handler
  CP(SCI_SETMODEVENTMASK, CP(SCI_GETMODEVENTMASK, 0, 0) or SC_MOD_CHANGEFOLD, 0);
  //----------------------------------------------------------------------------- 
  
  if _prop_Codepage <> 65001 then CP(SCI_STYLESETCHARACTERSET, 0, _prop_Codepage);
  
  //       
  fFontLstStyles.Text := _prop_Styles;  
  SetStyleLang(fFontLstStyles); //    
  //fFontLstStyles.Clear; //   ( ) 

  // ------------------- ----------------------------------------
  //    C,    , LexCPP.cxx
  if CP(SCI_GETLEXER, 0, 0) = 3 then    
    CP(SCI_SETPROPERTY, longint(PChar('lexer.cpp.track.preprocessor')), longint(PChar('0 ')));
    //CP(SCI_SETPROPERTY, longint(PChar('styling.within.preprocessor')), longint(PChar('1 ')));
  //    HTML,  , LexHTML.cxx
  if (CP(SCI_GETLEXER, 0, 0) = 4) or (CP(SCI_GETLEXER, 0, 0) = 5) then
    CP(SCI_SETPROPERTY, longint(PChar('fold.html')), longint(PChar('1 ')));   
 
  //  
  CP(SCI_SETPROPERTY, longint(PChar('fold')), longint(PChar('1 ')));
  CP(SCI_SETPROPERTY, longint(PChar('fold.comment')), longint(PChar('1 ')));
  CP(SCI_SETPROPERTY, longint(PChar('fold.compact')), longint(PChar('0 ')));
  CP(SCI_SETPROPERTY, longint(PChar('fold.preprocessor')), longint(PChar('1 ')));
  //-----------------------------------------------------------------------------
  
  //     
  CP(SCI_SETKEYWORDS, 0, longint(PChar(_prop_WordsKey1)));
  CP(SCI_SETKEYWORDS, 1, longint(PChar(_prop_WordsKey2)));
  CP(SCI_SETKEYWORDS, 2, longint(PChar(_prop_WordsKey3)));
  CP(SCI_SETKEYWORDS, 3, longint(PChar(_prop_WordsKey4)));
 
  //    
  if _prop_UserPoupMenu = 0 then CP(SCI_USEPOPUP, 0, 0);

  //     _prop_Strings
  CP(SCI_SETTEXT, 0, longint(PChar(_prop_Strings)));
  _prop_Strings := '';

  CP(SCI_EMPTYUNDOBUFFER, 0, 0); //   

  if _prop_Numbering then AutoSetMarginNumberWidth;

  //      (   ?)
  if _prop_ReadOnly = 0 then CP(SCI_SETREADONLY, 1, 0);

  // ------------------- ---------------------------------   
  //    1  -  ,   
  //CP(SCI_AUTOCSETCHOOSESINGLE, 1, 0); // 
  //    
  CP(SCI_AUTOCSETIGNORECASE, integer(_prop_IgnoreCase), 0);
  CP(SCI_AUTOCSETCASEINSENSITIVEBEHAVIOUR, integer(_prop_IgnoreCase), 0);    
  CP(SCI_AUTOCSETORDER, 1, 0);  //   
  //CP(SCI_AUTOCSETFILLUPS, 0, longint(PChar('( { '))); // ? 
  //-----------------------------------------------------------------------------    

  //        SCN_MODIFIED -
  // -    
  CP(SCI_SETMODEVENTMASK, SC_MOD_INSERTTEXT or SC_MOD_DELETETEXT, 0);

  //CP(SCI_SETVIEWWS, 2, 0); //  , 
  //CP(SCI_SETVIEWEOL, 1, 0);//   , 
  Control.Focused := true;
end;

procedure THIScintilla._work_doSave;
var
  fn, s: string;
  l: integer;
  
function SaveToFile(const FileName: AnsiString; Text: AnsiString): Boolean;
var F: HFile;
    Buf: AnsiString;
begin
  F := FileCreate(FileName, ofOpenWrite or ofCreateAlways);
  Result := F <> INVALID_HANDLE_VALUE;
  if Result then
  begin
    Buf := Text;
    FileWrite(F, Buf[1], Length(Buf));
    SetEndOfFile(F);
    FileClose(F);
  end;
end;
  
begin
  fn := ReadString(_Data, _data_FileName, _prop_FileName);
  l  := CP(SCI_GETTEXTLENGTH, 0, 0);
  SetLength(s, l + 1);
  CP(SCI_GETTEXT, l + 1, longint(PChar(s)));
  SaveToFile(fn, PChar(s));
  CP(SCI_SETSAVEPOINT, 0, 0);
  Control.Focused := true;
  _hi_OnEvent(_event_onSave);
end;

procedure THIScintilla._work_doLoad;
var
  fn, s: string;
begin
  fn := ReadString(_Data, _data_FileName, _prop_FileName);
  s  := StrLoadFromFile(fn);
  CP(SCI_SETTEXT, 0, longint(PChar(s)));
  CP(SCI_EMPTYUNDOBUFFER, 0, 0);
  Control.Focused := true;
  _hi_OnEvent(_event_onLoad);
end;

procedure THIScintilla._work_doStyles; //    
begin
  fFontLstStyles.Text := ToString(_Data);
  SetStyleLang(fFontLstStyles);
  //fFontLstStyles.Clear; //   
  Control.Focused := true;
end;

procedure THIScintilla._work_doAdd;
var
  s: string;
begin
  s := ReadString(_Data, _data_Str, '');
  if _prop_AddType = 0 then
    CP(SCI_ADDTEXT, Length(s), longint(PChar(s)))
  else
    CP(SCI_INSERTTEXT, 0, longint(PChar(s)));
end;

procedure THIScintilla._work_doClear;
begin
  CP(SCI_CLEARALL, 0, 0);
  _hi_OnEvent(_event_onClear);
end;

procedure THIScintilla._work_doText;
var
  s: string;
begin
  s := ToString(_Data);
  CP(SCI_SETTEXT, 0, longint(PChar(s)));
end;

procedure THIScintilla._work_doSetSelect;
var
  s: string;
begin
  s := ReadString(_Data, _data_str, '');
  CP(SCI_REPLACESEL, 0, longint(PChar(s)));
end;

procedure THIScintilla._work_doSetSelStart;
var
  pos: integer;
begin
  pos := ToInteger(_Data);
  fposition := pos;
  pos := CP(SCI_SETSELECTIONSTART, pos, 0);     
  pos := CP(SCI_SETSELECTIONEND, fposition, 0);
end;
 // lll
procedure THIScintilla._work_doSetSelLength;
var
  SelLength: integer; 
begin
  SelLength := ToInteger(_Data);

  
  CP(SCI_SETSEL, fposition, fposition + SelLength); 
end;

procedure THIScintilla._work_doSetZoom;
var
  dt: integer;
begin
  dt := ToInteger(_Data);
  if (dt > -10) or (dt < 20) then
  begin
    CP(SCI_SETZOOM, dt, 0);
    if _prop_Numbering then AutoSetMarginNumberWidth;
	if _prop_Markers then CP(SCI_SetMarginWidthN, 1, _prop_MarkMarginWidth + dt);
	if _prop_Fold then CP(SCI_SetMarginWidthN, 2, _prop_FoldMarginWidth + dt);
	Control.Focused := true;
  end;
end;

procedure THIScintilla._work_doEnsureVisible;
var
  pos: integer;
begin
  pos := ToInteger(_Data);
  fposition := pos;
  CP(SCI_GOTOPOS, pos, 0);
end;

procedure THIScintilla._work_doScrollBars;
var
  idx: integer;
begin
  idx := ToInteger(_Data);
  SetScrollBars(idx);
end;

procedure THIScintilla._work_doReadOnly;
var
  chk: integer;
begin
  chk := ToInteger(_Data);
  case chk of
    0: CP(SCI_SETREADONLY, 0, 0);
    1: CP(SCI_SETREADONLY, 1, 0);
  end;
end;

procedure THIScintilla._work_doSearch; 
var
  LineTarget, LinesScreen, LineFirst, PosSearchStart: longint;
begin
  strSearch := ReadString(_Data, _data_TextTarget, '');
  PosSearchStart := ToIntegerEvent(_data_PosSearchStart);
  if strSearch <> modSearch then    //    ,  
    posSearch := 0;
  modSearch := strSearch;

  //************************   ****************************************  
  if (posSearch = 0) and not pos0 then                   //   
  begin
    CP(SCI_SETTARGETSTART, PosSearchStart, 0);           //   
    CP(SCI_SETTARGETEND, CP(SCI_GETTEXTLENGTH, 0, 0), 0);//   
  end
  else
  if posSearch >= 0 then                                 //   
  begin
    CP(SCI_SETTARGETSTART, PosSearch + Length(strSearch), 0);
    CP(SCI_SETTARGETEND, CP(SCI_GETTEXTLENGTH, 0, 0), 0);
  end;

  //************************    **************************************  
  case ToIntegerEvent(_data_SearchFlags) of    
    0: CP(SCI_SETSEARCHFLAGS, 0, 0);    //       CP(SCI_SETSEARCHFLAGS, SCFIND_REGEXP, 0);   
    1: CP(SCI_SETSEARCHFLAGS, SCFIND_MATCHCASE, 0); //        
    2: CP(SCI_SETSEARCHFLAGS, SCFIND_WHOLEWORD, 0); //      
  end;

  //************************  **************************************************** 
  posSearch := CP(SCI_SEARCHINTARGET, Length(strSearch), longint(PChar(strSearch)));
  //  
  if posSearch = 0 then
    pos0 := True;  //    = 0,  

  //************************   *********************************
  if posSearch = -1 then
  begin
    _hi_onEvent(_event_onSearch, posSearch);
    posSearch := 0;
    pos0 := False;
    exit;
  end
  else
  begin
    LineTarget := CP(SCI_LINEFROMPOSITION, posSearch, 0) + 1; //     
    //    ,    (  )  
    if not boolean(CP(SCI_GETLINEVISIBLE, LineTarget, 0)) then
      CP(SCI_FOLDLINE, LineTarget, 1);
    LinesScreen := CP(SCI_LINESONSCREEN, 0, 0) - 1;     //       -1
    LineFirst := CP(SCI_GETFIRSTVISIBLELINE, 0, 0) + 1; //    

    //      Notepad++
    if (LineTarget > LineFirst) and (LineTarget <= (LineFirst + LinesScreen)) then
    begin
      CP(SCI_SETSEL, posSearch, posSearch + Length(strSearch));
    end
    else
    if LineTarget > (LineFirst + LinesScreen) then
    begin
      CP(SCI_GOTOLINE, LineTarget + LinesScreen div 2, 0);
      CP(SCI_SETSEL, posSearch, posSearch + Length(strSearch));
    end
    else
    begin
      CP(SCI_GOTOLINE, LineTarget - LinesScreen div 2, 0);
      CP(SCI_SETSEL, posSearch, posSearch + Length(strSearch));
    end;
    _hi_onEvent(_event_onSearch, posSearch);
  end;
end;

procedure THIScintilla._work_doCodepage;
var
  Param: integer;
begin
  Param := ToInteger(_Data);
  if (Param >= 0) and (Param < 21) then
  begin
	if ArrCodepage[Param] = 65001 then CP(SCI_SETCODEPAGE, 65001, 0) else CP(SCI_SETCODEPAGE, 0, 0);
	if ArrCodepage[Param] <> 65001 then 
	begin 
	  CP(SCI_STYLESETCHARACTERSET, 0, ArrCodepage[Param]);
	  _prop_Codepage:= ArrCodepage[Param];
	  SetStyleLang(fFontLstStyles);
	end;  
  end;
end;

procedure THIScintilla._work_doSelectBack;//  .
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_SETSELBACK, 1, lColor);
end;

procedure THIScintilla._work_doSelectFore;//   .
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_SETSELFORE, 1, lColor);
end;

procedure THIScintilla._work_doColor; //   
var
  lColor, i: integer;
begin
  lColor := ToInteger(_Data);
  for i := 0 to 32 do
    CP(SCI_STYLESETBACK, i, lColor);
end;

procedure THIScintilla._work_doFont; //  
var
  lFont: PFontRec;
  i: integer;
begin
  lFont := ToFont(_Data);
  for i := 0 to 255 do
  begin
    CP(SCI_STYLESETSIZE, i, lFont.size);                 // 
    CP(SCI_STYLESETFONT, i, longint(PChar(lFont.Name))); // 
  end;
  CP(SCI_STYLESETFORE, 1, lFont.Color);                  // 
  CP(SCI_STYLESETbold, 1, lFont.Style and 1);
  CP(SCI_STYLESETitalic, 1, lFont.Style and 2);
  CP(SCI_STYLESETunderline, 1, lFont.Style and 4);
  CP(SCI_STYLESETSIZE, STYLE_LINENUMBER, lFont.size +_prop_NumSizeCorrect);
  if _prop_Numbering then AutoSetMarginNumberWidth;
end;

procedure THIScintilla._work_doCaretlinevisible;
var
  wParam: integer;
begin
  wParam := ToInteger(_Data);
  CP(SCI_SETCARETLINEVISIBLE, wParam, 0);
  //   
  if CP(SCI_GETCARETLINEVISIBLE, 0, 0) = 0 then
    CP(SCI_STYLESETBACK, 34, _prop_Color)
  else
    CP(SCI_STYLESETBACK, 34, Styleback_34_35);

  if CP(SCI_GETCARETLINEVISIBLE, 0, 0) = 0 then
    CP(SCI_STYLESETBACK, 35, _prop_Color)
  else
    CP(SCI_STYLESETBACK, 35, Styleback_34_35);
end;

procedure THIScintilla._work_doCaretLineBack;
var
  wColor: integer;
begin
  wColor := ToInteger(_Data);
  CP(SCI_SETCARETLINEBACK, wColor, 0);
end;

procedure THIScintilla._work_doBraceHighlight;
begin
  CP(SCI_BRACEHIGHLIGHT, -1, -1);//   
  _prop_BraceHighlight := boolean(ToInteger(_Data));
end;

procedure THIScintilla._work_doBraceFore;
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_STYLESETFORE, 34, lColor);
end;

procedure THIScintilla._work_doBraceBold;
begin
  CP(SCI_STYLESETBOLD, 34, ToInteger(_Data));
end;

procedure THIScintilla._work_doBraceBadFore;
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_STYLESETFORE, 35, lColor);
end;

procedure THIScintilla._work_doBraceBadBold;
begin
  CP(SCI_STYLESETBOLD, 35, ToInteger(_Data));
end;

procedure THIScintilla._work_doIndentationGuides;
var
  wParam: integer;
begin
  wParam := ToInteger(_Data);
  CP(SCI_SETINDENTATIONGUIDES, wParam, 0);
end;

procedure THIScintilla._work_doColorIGFore;
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_STYLESETFORE, 37, lColor);
end;

procedure THIScintilla._work_doVLines;
var
  wParam: integer;
begin
  wParam := ToInteger(_Data);
  CP(SCI_SETEDGEMODE, wParam, 0);
end;

procedure THIScintilla._work_doLongVLines;
var
  wParam: integer;
begin
  wParam := ToInteger(_Data);
  CP(SCI_SETEDGECOLUMN, wParam, 0);
end;

procedure THIScintilla._work_doColorVLines;
var
  wColor: integer;
begin
  wColor := ToInteger(_Data);
  CP(SCI_SETEDGECOLOUR, wColor, 0);
end;

procedure THIScintilla._work_doColorOfFlags;
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_STYLESETFORE, STYLE_DEFAULT, lColor);
end;

procedure THIScintilla._work_doNumbering;
var
  Param: integer;
begin
  Param := ToInteger(_Data);
  case Param of
    0:
    begin
      _prop_Numbering := False;
      CP(SCI_SetMarginWidthN, 0, 0);
    end;
    1:
    begin
      _prop_Numbering := True;
      AutoSetMarginNumberWidth;
    end;
  end;
end;

procedure THIScintilla._work_doMarginNumFore;
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_STYLESETFORE, STYLE_LINENUMBER, lColor);
end;

procedure THIScintilla._work_doMarginNumBack;
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_STYLESETBACK, STYLE_LINENUMBER, lColor);
end;

procedure THIScintilla._work_doMarkers;
var
  Param: integer;
begin
  Param := ToInteger(_Data);
  case Param of
    0:
    begin
      _prop_Markers := False;
      CP(SCI_SetMarginWidthN, 1, 0);
      CP(SCI_MARKERDEFINE, 0, 5);
    end;
    1:
    begin
      _prop_Markers := True;
      CP(SCI_SetMarginWidthN, 1, 16);
      CP(SCI_MARKERDEFINE, 0, Markersymbol);
    end;
  end;
end;

procedure THIScintilla._work_doMarkMarginWidth;
var
  lParam: integer;
begin
  lParam := ToInteger(_Data);
  CP(SCI_SetMarginWidthN, 1, lParam);
end;

procedure THIScintilla._work_doMarkerForm;
var
  Param: integer;
begin
  Param := ToInteger(_Data);    
  if (Param >= 0) and (Param < 10) then
  begin
    CP(SCI_MARKERDEFINE, 0, ArrMarkerForm[Param]);
    Markersymbol := ArrMarkerForm[Param];
  end;
end;

procedure THIScintilla._work_doMarkerBack;
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_MARKERSETBACK, 0, lColor);
end;

procedure THIScintilla._work_doMarkerFore;
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_MARKERSETFORE, 0, lColor);
end;

procedure THIScintilla._work_doMarkerAdd;
begin
  CP(SCI_MARKERADD, (ToInteger(_Data) - 1), 0);
end;

procedure THIScintilla._work_doMarkerDelete;
begin
  CP(SCI_MARKERDELETE, (ToInteger(_Data) - 1), 0);
end;

procedure THIScintilla._work_doMarkerDeleteAll;
begin
  CP(SCI_MARKERDELETEALL, -1, 0);
end;

procedure THIScintilla._work_doGetLineMarker;
var
  i, StrCount: integer;
begin
  StrCount := CP(SCI_GETLINECOUNT, 0, 0);
  for i := 0 to StrCount - 1 do
  begin
    if CP(SCI_MARKERGET, i, 0) = 1 then
      _hi_onEvent(_event_onGetLineMarker, i + 1);
  end;
end;

procedure THIScintilla._work_doFold;
var
  Param: integer;
begin
  Param := ToInteger(_Data);
  case Param of
    0:
    begin
      _prop_Fold := False;
      CP(SCI_SetMarginWidthN, 2, 0);
      CP(SCI_FOLDALL, 1, 0);
    end;
    1:
    begin
      _prop_Fold := True;
      CP(SCI_SetMarginMaskN, 2, SC_MASK_FOLDERS);
      CP(SCI_SetMarginWidthN, 2, 16);
    end;
  end;
end;

procedure THIScintilla._work_doFoldMarginWidth;
var
  lParam: integer;
begin
  lParam := ToInteger(_Data);
  CP(SCI_SetMarginWidthN, 2, lParam);
end;

procedure THIScintilla._work_doFoldMarginColor1;
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_SETFOLDMARGINCOLOUR, 2, lColor);
end;

procedure THIScintilla._work_doFoldMarginColor2;
var
  lColor: integer;
begin
  lColor := ToInteger(_Data);
  CP(SCI_SETFOLDMARGINHICOLOUR, 2, lColor);
end;

procedure THIScintilla._work_doFoldIconFore;
var
  Color: integer;
begin
  Color := ToInteger(_Data);
  SetFoldIconColorFore(Color);
end;

procedure THIScintilla._work_doFoldIconBack;
var
  Color: integer;
begin
  Color := ToInteger(_Data);
  SetFoldIconColorBack(Color);
end;

procedure THIScintilla._work_doGetLineFold;
var
  i, StrCount: integer;
begin
  StrCount := CP(SCI_GETLINECOUNT, 0, 0);
  for i := 0 to StrCount - 1 do 
  begin
    if (CP(SCI_GETFOLDEXPANDED, i, 0) = 0) then _hi_onEvent(_event_onGetLineFold, i);
  end;
end;

procedure THIScintilla._work_doSetLineFold;
var
  i: integer;
  fArr: PArray;
  Item, eIndex: TData;
begin
  fArr := ReadArray(_data_ArrayFold);
  if fArr = nil then exit;
  for i := 0 to fArr._Count - 1 do
  begin
    dtInteger(eIndex, i);
    fArr._Get(eIndex, Item);
    CP(SCI_FOLDLINE , ToInteger(Item), 0);
  end;
end;

procedure THIScintilla._work_doFoldAll;
var
  wParam: integer;
begin
  wParam := ToInteger(_Data);
  CP(SCI_FOLDALL, wParam, 0);
  Control.Focused := true;
end;

procedure THIScintilla._work_doUndoRedo;
var
  Param: integer;
begin
  Param := ToInteger(_Data);
  case Param of
    0: CP(SCI_UNDO, 0, 0);
    1: CP(SCI_REDO, 0, 0);
  end;
  Control.Focused := true;
end;

procedure THIScintilla._work_doEditingSelected;
var
  Param: integer;
begin
  Param := ToInteger(_Data);
  case Param of
    0: CP(SCI_CUT, 0, 0);
    1: CP(SCI_COPY, 0, 0);
    2: CP(SCI_PASTE, 0, 0);
    3: CP(SCI_CLEAR, 0, 0);
    4: CP(SCI_SELECTALL, 0, 0);
  end;
  Control.Focused := true;
end;

procedure THIScintilla._work_doSetEmptyUndoBuffer;
begin
  CP(SCI_EMPTYUNDOBUFFER, 0, 0);
end;
 
procedure THIScintilla._work_doGetStatus;
begin
  _hi_OnEvent(_event_onError, CP(SCI_GETSTATUS, 0, 0));
end;
 
procedure THIScintilla._work_doWrapMode;
var
  wParam: integer;
begin
  wParam := ToInteger(_Data);
  if ((CP(SCI_GETHSCROLLBAR, 0, 0) = 0) and (CP(SCI_GETVSCROLLBAR, 0, 0) = 1)) or
     ((CP(SCI_GETHSCROLLBAR, 0, 0) = 0) and (CP(SCI_GETVSCROLLBAR, 0, 0) = 0)) then
       CP(SCI_SETWRAPMODE, wParam + 1, 0);
end;

procedure THIScintilla._work_doWrapVisualFlags;
var
  Param: integer;
begin
  Param := ToInteger(_Data);
  case Param of
    0: CP(SCI_SETWRAPVISUALFLAGS, 0, 0);
    1: CP(SCI_SETWRAPVISUALFLAGS, SC_WRAPVISUALFLAG_START or SC_WRAPVISUALFLAG_END, 0);
  end;
end;

procedure THIScintilla._work_doAutoComplete;
var
  Param: integer;
begin
  Param := ToInteger(_Data);
  case Param of
    0: _prop_AutoComplete := False;
    1: _prop_AutoComplete := True;
  end;
end;

procedure THIScintilla._work_doIgnoreCase;
var
  wParam: integer;
begin
  wParam := ToInteger(_Data);
  CP(SCI_AUTOCSETIGNORECASE, wParam, 0);
  CP(SCI_AUTOCSETCASEINSENSITIVEBEHAVIOUR, wParam, 0);
end;

procedure THIScintilla._work_doAutoCompStrings;
var
  Param: string;
begin
  Param := ToString(_Data);
  _prop_AutoCompStrings := Param;
end;

procedure THIScintilla._work_doMinWordLen;
var
  Param: integer;
begin
  Param := ToInteger(_Data);
  _prop_MinWordLen := Param;
end;
// lll
////////////////////////////////////   fLength  fposition
procedure THIScintilla._work_doIndicators; //    ,  
var
  Param, Wend: integer;
{begin
  Param := ToInteger(_Data);
 // Param := CP(SCI_WORDSTARTPOSITION ,Param,1);
  Wend:=CP(SCI_GETSELECTIONEND,0,0);
 // Wend:=CP(SCI_WORDENDPOSITION ,Wend,1);
  CP(SCI_INDICATORFILLRANGE,Param,Wend - Param);   
end;  }
begin
 
//  Param := ToInteger(_Data);    //
  Param := CP(SCI_GETSELECTIONSTART ,Param,1);  
  Wend:=CP(SCI_GETSELECTIONEND,Param,0);  //
  Param := CP(SCI_WORDSTARTPOSITION ,Param,1);
  Wend:=CP(SCI_WORDENDPOSITION ,Wend,1);  // SCI_SETWORDCHARS 
  CP(SCI_INDICATORFILLRANGE,Param,Wend - Param); // 
end;                         

procedure THIScintilla._work_doIndicStyle;
var
  Param: integer;
begin
    Param:= ToInteger(_Data);
   _prop_IndicStyle := ArrIndicForm[Param];
end;

procedure THIScintilla._work_doIndicFore;
begin   
  _prop_IndicFore := ToInteger(_Data); 
end;

procedure THIScintilla._work_doIndicatorAdd;
var
  Param: integer;  
begin
Param := ToInteger(_Data);
  CP(SCI_INDICSETSTYLE, Param, _prop_IndicStyle);
  CP(SCI_INDICSETFORE, Param, _prop_IndicFore);    
end;

procedure THIScintilla._work_doSetIndicatorCurrent;
var
  Param: integer;    
begin
Param := ToInteger(_Data);
CP(SCI_SETINDICATORCURRENT, Param, 0); 
end;

procedure THIScintilla._work_doIndicatorClear;
var
indic, Param: integer;
begin
Param := ToInteger(_Data);
   for indic := 0 To 31 do
   begin
   CP(SCI_SETINDICATORCURRENT, indic, 0);
   CP(SCI_INDICATORCLEARRANGE ,0,CP(SCI_GETTEXTLENGTH, 0, 0)-1);
  end;
end;
/////////////////////////////////////// 
procedure THIScintilla._work_doGetLineMarkerNext;  //   
var
oldnum,line, pos, StrCount: integer;
begin
  StrCount := CP(SCI_GETLINECOUNT, 0, 0);
  if StrCount = 0 then exit;
  pos  := CP(SCI_GETCURRENTPOS, 0, 0);
  line := CP(SCI_LINEFROMPOSITION, pos, 0);
  oldnum := line;
       case  ToInteger(_Data) of
 0:
   begin
   line := -1;
      repeat
    inc(line); if line= oldnum then exit;
	if line = StrCount then line:=-1; 
   until CP(SCI_MARKERGET, line, 0) = 1;
   end;
 1:
 begin      
   repeat
    inc(line); if line= oldnum then exit;
	if line = StrCount then line:=-1; 
   until CP(SCI_MARKERGET, line, 0) = 1;
  //  CP(SCI_GOTOLINE, line, 0);
end;
-1:
   begin
         repeat
    dec(line); if line = oldnum then exit;
	if line < 0 then line:=StrCount; 
   until CP(SCI_MARKERGET, line, 0) = 1;   
   end;  
     end;
    CP(SCI_GOTOLINE, line, 0);
end; 

procedure THIScintilla._work_doAnnotationSetText; 
var
line: integer;
S: string;  
begin
line := ToInteger(_Data)-1;
S := ToStringEvent(_data_TextAnnotation);
CP(SCI_ANNOTATIONSETTEXT,line, longint(pchar(S)));
CP(SCI_ANNOTATIONSETSTYLE,line, 25);
CP(SCI_ANNOTATIONSETVISIBLE, _prop_AnnotationStyle,0);

//CP(SCI_SETMARGINTYPEN,1,SC_MARGIN_TEXT);
//CP(SCI_SETMARGINWIDTHN,1,80);
//CP( SCI_MARGINSETSTYLEOFFSET ,  200 , 0); 
//CP( SCI_STYLESETBACK ,  200 , $CCD5D9 );
//CP(SCI_MARGINSETTEXT,4, longint(pchar('qwerty'))); 
end;
procedure THIScintilla._work_doAnnotationBack;
begin
_prop_AnnotationBack := ToInteger(_Data);
end;
procedure THIScintilla._work_doAnnotationFore;
begin
_prop_AnnotationFore := ToInteger(_Data);
end;
procedure THIScintilla._work_doAnnotationStyle;
var
  Param: integer;
begin
Param := ToInteger(_Data);
_prop_AnnotationStyle := ArrAnnotationStyle[Param];
CP(SCI_ANNOTATIONSETVISIBLE, _prop_AnnotationStyle,0);
end;
procedure THIScintilla._work_doAnnotationStrDel;
var
  Param: integer;
begin
Param := ToInteger(_Data)-1;
CP(SCI_ANNOTATIONSETTEXT,Param, 0 );
end;
procedure THIScintilla._work_doAnnotationClearAll;
begin
CP(SCI_ANNOTATIONCLEARALL,0,0 );
end;
////////////////////////
  
procedure THIScintilla._var_Count;
var
  l: integer;
begin
  l := CP(SCI_GETLINECOUNT, 0, 0);
  dtInteger(_Data, l);
end;

procedure THIScintilla._var_Text;
var
  s: string;
  l: integer;
begin
  l := CP(SCI_GETTEXTLENGTH, 0, 0);
  SetLength(s, l + 1);
  CP(SCI_GETTEXT, l + 1, longint(PChar(s)));
  dtString(_Data, PChar(s));
end;

procedure THIScintilla._Set;
var
  ind, l: integer;
  s: string;
begin
  ind := ToIntIndex(Item);
  //l := CP(SCI_GETLINEENDPOSITION,ind,0);
  l := CP(SCI_POSITIONFROMLINE, ind, 0);
  flinecount := CP(SCI_GETLINECOUNT, 0, 0);
  if (ind >= 0) and (ind < flinecount) then
  begin
    s := ToString(Val);
    s := s + #13#10;
    CP(SCI_INSERTTEXT, l, longint(PChar(s)));
  end;
end;

procedure THIScintilla._Add;
var
  s: string;
begin
  s := ToString(val);
  CP(SCI_ADDTEXT, Length(s), longint(PChar(s)));
end;

function THIScintilla._Get;
var
  ind, l: integer;
  s: string;
begin
  Result := False;
  ind := ToIntIndex(Item);
  flinecount := CP(SCI_GETLINECOUNT, 0, 0);
  if (ind >= 0) and (ind < flinecount) then
  begin
    Result := True;
    l := CP(SCI_LINELENGTH, ind, 0);
    SetLength(s, l);
    CP(SCI_GETLINE, ind, longint(PChar(s)));
    dtString(Val, s);
  end;
end;

function THIScintilla._Count;
begin
  flinecount := CP(SCI_GETLINECOUNT, 0, 0);
  Result := flinecount;
end;

procedure THIScintilla._var_Array;
begin
  if Arr = nil then
    Arr := CreateArray(_Set, _Get, _Count, _Add);
  dtArray(_Data, Arr);
end;

procedure THIScintilla._var_Position;
var
  pos: integer;
begin
  pos := CP(SCI_GETCURRENTPOS, 0, 0);
  fposition := pos;
  dtInteger(_Data, pos);
end;

procedure THIScintilla._var_SelText;
var
  s: string;
  l: integer;
begin
  l := CP(SCI_GETSELECTIONEND, 0, 0);
  SetLength(s, l);
  CP(SCI_GETSELTEXT, 0, longint(PChar(s)));
  dtString(_Data, s);
end;

procedure THIScintilla._var_LinePosition;
var
  line, pos: integer;
begin
  pos  := CP(SCI_GETCURRENTPOS, 0, 0);
  line := CP(SCI_LINEFROMPOSITION, pos, 0);
  dtInteger(_Data, line + 1);
end;

procedure THIScintilla._var_StrPosition;
var
  pos, pos1: integer;
begin
  pos  := CP(SCI_GETCURRENTPOS, 0, 0);
  pos1 := CP(SCI_GETCOLUMN, pos, 0);
  dtInteger(_Data, pos1 + 1);
end;

procedure THIScintilla._var_TextLength;
var
  len: integer;
begin
  len := CP(SCI_GETTEXTLENGTH, 0, 0);
  dtInteger(_Data, len);
end;

procedure THIScintilla._var_GetModify;
var
  status: integer;
begin
  status := CP(SCI_GETMODIFY, 0, 0);
  dtInteger(_Data, status);
end;

//initialization

//finalization

end.